#
# Copyright (C) 2007, 2008 Red Hat, Inc.
# Authors:
# Thomas Woerner <twoerner@redhat.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import sys, types

from fw_config import *
from fw_functions import getPortID, getServiceName, getPortRange
import fw_services

from snack import *

class ui:
    def init(self, config):
        self.config = config

        other_ports = [ ]
        if config.ports:
            for entry in config.ports:
                if len(entry[0]) > 1:
                    other_ports.append("%s-%s:%s" % (entry[0][0], entry[0][1],
                                                     entry[1]))
                else:
                    other_ports.append("%s:%s" % (entry[0][0], entry[1]))

        custom_rules = [ ]
        if config.custom_rules:
            for entry in config.custom_rules:
                custom_rules.append(':'.join(entry))

        self.screen = SnackScreen()
        self.screen.drawRootText(0, 0, APP_NAME)
        #self.screen.popHelpLine()

        self.toplevel = GridForm(self.screen, _("Firewall Configuration"),
                                 1, 3)
        tr = TextboxReflowed(
            50, _("A firewall protects against unauthorized "
                  "network intrusions. Enabling a firewall blocks all "
                  "incoming connections. Disabling a firewall allows "
                  "all connections and is not recommended. "))
        self.toplevel.add(tr, 0, 0, padding=(0,0,0,1), growx=1)
        grid = Grid(3, 1)
        grid.setField(Label(_("Firewall:")), 0, 0, padding=(0,0,1,0))
        if not config.filename:
            enabled = 0
        else:
            enabled = int(self.config.enabled)
        self.enabled = SingleRadioButton(_("Enabled"), None, enabled)
        grid.setField(self.enabled, 1, 0, padding=(0,0,1,0))
        grid.setField(SingleRadioButton(_("Disabled"), self.enabled,
                                        1-enabled), 2, 0)
        self.toplevel.add(grid, 0, 1, padding=(0,0,0,1))
        self.toplevel.bb = ButtonBar(self.screen,
                                     ((_("OK"), "ok"),
                                      (_("Customize"), "customize"),
                                      (_("Cancel"), "cancel")))
        self.toplevel.add(self.toplevel.bb, 0, 2, growx=1)

        self.customize = GridForm(self.screen,
                                  _("Firewall Configuration - Customize"),
                                  1, 6)
        self.customize.add(TextboxReflowed(
            65, _("You can customize your firewall in two ways. "
                  "First, you can select to allow all traffic from "
                  "certain network interfaces. Second, you can allow "
                  "certain protocols explicitly through the firewall. "
                  "Specify additional ports in the form 'service:protocol', "
                  "such as 'imap:tcp'. ")), 0, 1, padding=(0,0,0,1), growx=1)
        grid = Grid(3, 1)

        t_grid = Grid(1, 2)
        t_grid.setField(Label(_("Trusted Services:")), 0, 0, padding=(0,0,1,0),
                        anchorLeft=1, anchorTop=1)
        self.services = CheckboxTree(4, scroll=1)
        keys = [ svc.name for svc in fw_services.service_list ]
        keys.sort()

        for key in keys:
            svc = fw_services.getByName(key)
            enabled = False
            if svc.key in config.services:
                enabled = True
            self.services.append(svc.name, selected=enabled)
        t_grid.setField(self.services, 0, 1, anchorLeft=1, anchorTop=1)
        grid.setField(t_grid, 0, 0, padding=(0,0,1,1), anchorLeft=1,
                      anchorTop=1)

        t_grid = Grid(1, 2)
        t_grid.setField(Label(_("Trusted Interfaces:")), 0, 0,
                        padding=(0,0,1,0), anchorLeft=1, anchorTop=1)
        self.trust = CheckboxTree(4, scroll=1)
        devices = [ ]
        for dev in STD_DEVICES:
            devices.append(dev+"+")
        try:
            for dev in NCDeviceList.getDeviceList():
                if dev.Alias and dev.Alias != "":
                    # ignore device aliases: not usable for iptables
                    continue
                if not dev.Device in devices:
                    devices.append(dev.Device)
        except:
            pass
        devices.sort()
        matched = [ ]
        for dev in devices:
            enabled = False
            if config.trust and dev in config.trust:
                enabled = True
                matched.append(dev)
            self.trust.append(dev, selected=enabled)
        if config.trust:
            for dev in config.trust:
                if not dev in matched:
                    self.trust.append(dev, selected=True)
        t_grid.setField(self.trust, 0, 1, anchorLeft=1, anchorTop=1)
        grid.setField(t_grid, 1, 0, padding=(0,0,1,1), anchorLeft=1,
                      anchorTop=1)

        t_grid = Grid(1, 2)
        t_grid.setField(Label(_("Masquerading:")), 0, 0,
                        padding=(0,0,1,0), anchorLeft=1, anchorTop=1)
        self.masq = CheckboxTree(4, scroll=1)
        matched = [ ]
        for dev in devices:
            enabled = False
            if config.masq and dev in config.masq:
                enabled = True
                matched.append(dev)
            self.masq.append(dev, selected=enabled)
        if config.masq:
            for dev in config.masq:
                if not dev in matched:
                    self.masq.append(dev, selected=True)
        t_grid.setField(self.masq, 0, 1, anchorLeft=1, anchorTop=1)
        grid.setField(t_grid, 2, 0, padding=(0,0,1,1), anchorLeft=1,
                      anchorTop=1)

        self.customize.add(grid, 0, 2, growx=1)

        grid = Grid(2, 1)

        t_grid = Grid(1, 2)
        t_grid.setField(Label(_("Other Ports:")), 0, 0,
                        padding=(0,0,1,0), anchorLeft=1, anchorTop=1)
        self.other_ports = Entry(25, text=' '.join(other_ports))
        t_grid.setField(self.other_ports, 0, 1, anchorLeft=1, anchorTop=1)
        grid.setField(t_grid, 0, 0, padding=(0,0,1,1), anchorLeft=1,
                      anchorTop=1)

        t_grid = Grid(1, 2)
        t_grid.setField(Label(_("Custom Rules:")), 0, 0,
                        padding=(0,0,1,0), anchorLeft=1, anchorTop=1)
        self.custom_rules = Entry(40, text=' '.join(custom_rules))
        t_grid.setField(self.custom_rules, 0, 1, anchorLeft=1, anchorTop=1)
        grid.setField(t_grid, 1, 0, padding=(0,0,1,1), anchorLeft=1,
                      anchorTop=1)

        self.customize.add(grid, 0, 3, growx=1)

        self.customize.bb = ButtonBar(self.screen, ((_("OK"), "ok"),))
        self.customize.add(self.customize.bb, 0, 5, growx=1)

    def finish(self):
        self.screen.finish()

    def apply(self):
        conf = self.config
        conf.enabled = self.enabled.selected()

        # trusted services
        conf.ports = [ ]
        conf.add_module = [ ]
        conf.remove_module = [ ]
        for svc in fw_services.service_list:
            if svc.name in self.services.getSelection():
                if svc.key not in conf.services:
                    conf.services.append(svc.key)
                conf.add_module.extend(svc.modules)
            else:
                if svc.key in conf.services:
                    conf.services.remove(svc.key)
                conf.remove_module.extend(svc.modules)

        # trusted interfaces
        conf.trust = self.trust.getSelection()

        # masquerading
        conf.masq = self.masq.getSelection()

        # other ports
        ports = self.other_ports.value()
        if ports and ports != "":
            for entry in ports.split(" "):
                val = entry.strip().split(":")
                if len(val) != 2:
                    # TODO: error dialog
                    continue
                (port, proto) = val
                range = getPortRange(port.strip())
                conf.ports.append((range, proto))

        # custom rules
        conf.custom_rules = [ ]
        val = self.custom_rules.value()
        if val and val != "":
            for value in val.split():
                type = "ipv4"
                table = "filter"
                splits = value.split(":", 1)
                if len(splits) > 1 and splits[0] in FIREWALL_TYPES:
                    type = splits[0]
                    splits = splits[1].split(":", 1)
                if len(splits) > 1 and splits[0] in FIREWALL_TABLES:
                    table = splits[0]
                    splits = splits[1].split(":", 1)
                filename = ":".join(splits)
                conf.custom_rules.append((type, table, filename))

    def parse_error(self, msg):
        print >>sys.stderr, msg

    def parse_exit(self, status=0):
        sys.exit(status)

    def main(self):
        while 1:
            res = self.toplevel.runPopup()
            str = self.toplevel.bb.buttonPressed(res)
            if str == 'ok':
                self.apply()
                return True
            elif str == 'cancel':
                return False
            elif str == 'customize':
                while 1:
                    res2 = self.customize.runPopup()
                    str = self.customize.bb.buttonPressed(res2)
                    if str == 'ok':
                        self.apply()
                        self.finish()
                        self.init(self.config)
                        break

    def run(self, config):
        self.init(config)
        res = self.main()
        self.apply()
        self.finish()
        return res
